from ftw.globalstatusmessage.interfaces import IStatusMessageConfigForm
from ftw.publisher.core import states
from ftw.publisher.core.communication import createResponse
from ftw.publisher.core.utils import encode_after_json
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from zope.component import getUtility
from zope.schema import getFieldNames
import json


class ConfigReceiverView(BrowserView):

    def __call__(self):
        data = encode_after_json(
            json.loads(self.request.form.get('jsondata'))
        )

        if not data:
            return createResponse(states.InvalidRequestError())

        registry = getUtility(IRegistry)
        settings = registry.forInterface(IStatusMessageConfigForm, check=False)

        for field_name in getFieldNames(IStatusMessageConfigForm):
            if field_name in data:
                setattr(settings, field_name, data[field_name])

        return createResponse(states.SuccessState())



# -*- coding: utf-8 -*-
from Acquisition._Acquisition import aq_inner
from ftw.globalstatusmessage import _
from ftw.globalstatusmessage.interfaces import IStatusMessageConfigForm
from ftw.globalstatusmessage.utils import is_profile_installed
from plone import api
from plone.app.registry.browser import controlpanel
from plone.registry.interfaces import IRegistry
from Products.statusmessages import STATUSMESSAGEKEY
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from zope.annotation import IAnnotations
from zope.component import getUtility
from zope.i18nmessageid import MessageFactory
from zope.schema import getFieldNames
import json
import pkg_resources

try:
    pkg_resources.get_distribution('ftw.publisher.sender')
except pkg_resources.DistributionNotFound:
    pass
else:
    from ftw.publisher.sender.interfaces import IConfig
    from ftw.publisher.sender.utils import sendJsonToRealm
    from ftw.publisher.core.utils import decode_for_json


plone_messagefactory = MessageFactory("plone")


class StatusMessageEditForm(controlpanel.RegistryEditForm):
    schema = IStatusMessageConfigForm
    label = _(u'Global Status Message')
    description = _(u'Settings for Global Status Message.')


class PublishingStatusMessageEditForm(StatusMessageEditForm):
    """
    A special edit form which handles the publishing of the global status message
    to a "ftw.publisher.receiver" instance.

    Please note that the buttons are rendered in the order as they are defined.
    """

    @button.buttonAndHandler(plone_messagefactory(u"Save"), name='save')
    def handleSave(self, action):
        # Redeclare the save button because it is not inherited from the parent class.
        super(PublishingStatusMessageEditForm, self).handleSave(self, action)

    @button.buttonAndHandler(_(u'Save and publish'), name='save_and_publish')
    def handleSaveAndPublish(self, action):
        """
        The additional button to send to a "ftw.publisher.receiver" instance.
        """
        self.handleSave(self, action)

        self.send_to_receiver()

        # Consume Plone messages set by `handleSave` so we can set our own message.
        messages = IStatusMessage(self.request)
        if messages.show():
            self.request.response.cookies.pop(STATUSMESSAGEKEY)
            IAnnotations(self.request)[STATUSMESSAGEKEY] = None

        # Set a new Plone message.
        api.portal.show_message(
            message=_(u'Changes saved and published.'),
            request=self.request,
            type='info'
        )

    def updateActions(self):
        super(PublishingStatusMessageEditForm, self).updateActions()
        self.actions['save_and_publish'].addClass("context")

    @button.buttonAndHandler(plone_messagefactory(u'Cancel'), name='cancel')
    def handleCancel(self, action):
        # Redeclare the cancel button because it is not inherited from the parent class.
        super(PublishingStatusMessageEditForm, self).handleCancel(self, action)

    def send_to_receiver(self):
        data = json.dumps(
            decode_for_json(self.get_settings_data())
        )

        realms = IConfig(api.portal.get()).getRealms()
        for realm in realms:
            sendJsonToRealm(
                data,
                realm,
                'global_statusmessage_config_receiver'
            )

    def get_settings_data(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IStatusMessageConfigForm, check=False)

        data = {
            field_name: getattr(settings, field_name)
            for field_name in getFieldNames(IStatusMessageConfigForm)
        }

        return data


class StatusMessageControlPanel(controlpanel.ControlPanelFormWrapper):
    form = StatusMessageEditForm

    def __init__(self, context, request):
        super(StatusMessageControlPanel, self).__init__(context, request)

        if is_profile_installed('profile-ftw.publisher.sender:default'):
            self.form_instance = PublishingStatusMessageEditForm(
                aq_inner(context), request
            )
            self.form_instance.__name__ = self.__name__



import re
from plone import api


def is_path_included(path, included_paths, excluded_paths):
    """Helper function for testing whether a path is in within any
    container in included_paths or in excluded_paths.

    The nearest container path wins.
    If the path is not in any of included_paths or excluded_paths,
    ``True`` is returned as default.
    """
    sites = {'/': True}
    sites.update(dict((path, True) for path in included_paths))
    sites.update(dict((path, False) for path in excluded_paths))

    matching_sites = [site_path for site_path in sites.keys()
                      if path.startswith(site_path)]
    matching_sites.sort()
    return sites[matching_sites[-1]]


def is_profile_installed(profileid):
    """
    Checks whether a generic setup profile is installed.
    Respects product uninstallation via quickinstaller.

    Inspired by `ftw.upgrade.step.UpgradeStep#is_profile_installed`.
    """
    profileid = re.sub(r'^profile-', '', profileid)
    portal_setup = api.portal.get_tool('portal_setup')

    try:
        profileinfo = portal_setup.getProfileInfo(profileid)
    except KeyError:
        return False

    if not is_product_installed(profileinfo['product']):
        return False

    version = portal_setup.getLastVersionForProfile(profileid)
    return version != 'unknown'


def is_product_installed(product_name):
    """
    Check whether a product is installed.

    Inspired by `ftw.upgrade.step.UpgradeStep#is_product_installed`.
    """
    quickinstaller = api.portal.get_tool('portal_quickinstaller')
    return quickinstaller.isProductInstallable(product_name) and \
        quickinstaller.isProductInstalled(product_name)



# -*- coding: utf-8 -*-
from ftw.globalstatusmessage.config import PROJECTNAME
from ftw.globalstatusmessage.interfaces import IStatusMessageConfigForm
from ftw.globalstatusmessage.testing import STATUSMESSAGE_FUNCTIONAL
from ftw.globalstatusmessage.tests import FunctionalTestCase
from ftw.globalstatusmessage.tests.helpers import view_registered
from ftw.publisher.core.communication import createResponse
from ftw.publisher.core.states import SuccessState
from ftw.publisher.core.utils import decode_for_json
from ftw.publisher.core.utils import encode_after_json
from ftw.publisher.sender.interfaces import IConfig
from ftw.publisher.sender.persistence import Realm
from ftw.testbrowser import browsing
from ftw.testbrowser.pages.statusmessages import info_messages
from plone import api
from plone.app.testing import logout, SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five import BrowserView
from zope.component import getMultiAdapter, getUtility
from zope.interface import Interface
from zope.schema import getFieldNames
import json
import transaction
import unittest2 as unittest


class ControlPanelTestCase(unittest.TestCase):

    layer = STATUSMESSAGE_FUNCTIONAL

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.controlpanel = self.portal['portal_controlpanel']

    def test_controlpanel_has_view(self):
        view = getMultiAdapter(
            (self.portal, self.request), name='global_statusmessage_config')
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@global_statusmessage_config')

    def test_controlpanel_installed(self):
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertIn('globalstatusmessage', actions)

    def test_controlpanel_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']
        qi.uninstallProducts(products=[PROJECTNAME])
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertNotIn('globalstatusmessage', actions)


class PublishedControlPanelTestCase(FunctionalTestCase):

    enabled_bool = True
    enabled_anonymous_bool = True
    type_choice = u'information'
    title_textfield = u'Hello World'
    message_textfield = u'This is the message to the world'
    exclude_sites = None

    form_data = {
        'Active': enabled_bool,
        'Show to anonymous users?': enabled_anonymous_bool,
        'Type': type_choice,
        'Title': title_textfield,
        'Message': message_textfield,
    }

    expected_settings = {
        'enabled_bool': enabled_bool,
        'enabled_anonymous_bool': enabled_anonymous_bool,
        'type_choice': type_choice,
        'title_textfield': title_textfield,
        'message_textfield': message_textfield,
        'exclude_sites': exclude_sites,
    }

    payload = json.dumps(decode_for_json(expected_settings))

    def setUp(self):
        super(PublishedControlPanelTestCase, self).setUp()
        self.grant('Manager')
        api.portal.get_tool('portal_setup').runAllImportStepsFromProfile(
            'profile-ftw.publisher.sender:default',
            ignore_dependencies=True
        )

    def test_all_settings_are_tested(self):
        """
        This test will fail if new settings are added to "IStatusMessageConfigForm"
        without having been added in this test case.
        """
        self.assertEqual(
            set(self.expected_settings.keys()),
            set(getFieldNames(IStatusMessageConfigForm)),
            msg='Have you added some fields to "IStatusMessageConfigForm" without updating the test case?'
        )

    @browsing
    def test_sender(self, browser):
        intercepted_data = {}

        class MockedReceiverView(BrowserView):
            def __call__(self):
                intercepted_data['jsondata'] = self.request.form.get('jsondata')
                return createResponse(SuccessState())

        config = IConfig(self.portal)
        config.appendRealm(Realm(1, self.portal.absolute_url(), SITE_OWNER_NAME, SITE_OWNER_PASSWORD))
        transaction.commit()

        with view_registered(MockedReceiverView, 'global_statusmessage_config_receiver',
                             required=(IPloneSiteRoot, Interface)):
            browser.login(SITE_OWNER_NAME).open(view='@@global_statusmessage_config')
            browser.fill(self.form_data)
            browser.click_on('Save and publish')

        self.assertDictEqual(
            self.expected_settings,
            encode_after_json(json.loads(intercepted_data['jsondata']))
        )

        self.assertEqual(
            ['Changes saved and published.'],
            info_messages()
        )

    @browsing
    def test_receiver(self, browser):
        browser.login().open(
            self.portal,
            view='@@global_statusmessage_config_receiver',
            data={'jsondata': self.payload},
            send_authenticator=True)

        registry = getUtility(IRegistry)
        settings = registry.forInterface(IStatusMessageConfigForm, check=False)

        self.assertDictEqual(
            {
                field_name: getattr(settings, field_name)
                for field_name in getFieldNames(IStatusMessageConfigForm)
            },
            self.expected_settings
        )



from contextlib import contextmanager
from zope.browser.interfaces import IBrowserView
from zope.component import getGlobalSiteManager
from zope.interface import Interface


@contextmanager
def view_registered(factory, name, required=(Interface, Interface)):
    options = dict(
        factory=factory,
        required=required,
        provided=IBrowserView,
        name=name,
    )
    getGlobalSiteManager().registerAdapter(**options)
    try:
        yield
    finally:
        getGlobalSiteManager().unregisterAdapter(**options)



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import applyProfile, PLONE_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing.z2 import installProduct
from plone.testing.z2 import ZSERVER_FIXTURE
from Products.CMFPlone.utils import getFSVersionTuple
from zope.configuration import xmlconfig
import logging
import sys

handler = logging.StreamHandler(sys.stderr)
logging.root.addHandler(handler)


class ZCMLLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, ZSERVER_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        installProduct(app, 'ftw.globalstatusmessage')

STATUSMESSAGE_ZCML_LAYER = ZCMLLayer()
STATUSMESSAGE_ZCML_FUNCTIONAL = FunctionalTesting(
    bases=(STATUSMESSAGE_ZCML_LAYER, ),
    name="ftw.globalstatusmessage:zcml:functional")


class InstallationLayer(PloneSandboxLayer):

    defaultBases = (STATUSMESSAGE_ZCML_LAYER, BUILDER_LAYER)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.globalstatusmessage:default')

        if getFSVersionTuple() > (5, ):
            applyProfile(portal, 'plone.app.contenttypes:default')

STATUSMESSAGE_FIXTURE = InstallationLayer()
STATUSMESSAGE_FUNCTIONAL = FunctionalTesting(
    bases=(STATUSMESSAGE_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.globalstatusmessage:functional")
